# Generated by Django 3.1.1 on 2021-05-10 13:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0002_oidcsession_salt'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='oidcissuedtoken',
            options={'verbose_name': 'Issued Token',
                     'verbose_name_plural': 'Issued Tokens'},
        ),
    ]
