# Generated by Django 3.1.1 on 2021-05-16 12:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0006_auto_20210515_1001'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oidcissuedtoken',
            name='value',
            field=models.TextField(unique=True),
        ),
    ]
