# Generated by Django 3.1.1 on 2021-06-08 17:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0010_remove_oidcsession_session_info'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oidcissuedtoken',
            name='type',
            field=models.CharField(choices=[('authorization_code', 'authorization_code'), (
                'access_token', 'access_token'), ('id_token', 'id_token'), ('refresh_token', 'refresh_token')], max_length=32),
        ),
    ]
