<!DOCTYPE html>
<head>
    <meta charset="utf-8">
    <title>Logout Request</title>
    <meta name="viewport"
          content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <meta http-equiv="x-ua-compatible" content="ie=edge">
    <style>
        @import url(https://fonts.googleapis.com/css?family=Roboto:400,100);

        button, h1 {
            text-align: center
        }

        h1 {
            font-weight: 100;
            font-size: 1.3em
        }

        body {
            font-family: Roboto, sans-serif;
            margin-top: 25px;
            margin-bottom: 25px
        }

        .container {
            padding: 0 40px 10px;
            width: 274px;
            background-color: #F7F7F7;
            margin: 0 auto 10px;
            border-radius: 2px;
            box-shadow: 0 2px 2px rgba(0, 0, 0, .3);
            overflow: hidden
        }

        button {
            font-size: 14px;
            font-family: Arial, sans-serif;
            font-weight: 700;
            height: 36px;
            padding: 0 8px;
            width: 100%;
            display: block;
            margin-bottom: 10px;
            position: relative;
            border: 0;
            color: #fff;
            text-shadow: 0 1px rgba(0, 0, 0, .1);
            background-color: #4d90fe;
            cursor: pointer
        }

        button:hover {
            border: 0;
            text-shadow: 0 1px rgba(0, 0, 0, .3);
            background-color: #357ae8
        }
    </style>
</head>
<body>
<div class="container">
    <h1>Do you want to sign-out from {{ op }}?</h1>
    <script>
        function logout() {
            var form = document.getElementById('op.logoutForm');
            var input = document.createElement('input');
            input.type = 'hidden';
            input.name = 'logout';
            input.value = 'yes';
            form.appendChild(input);
            form.submit();
        }

        function rpLogoutOnly() {
            var form = document.getElementById('op.logoutForm');
            form.submit();
        }
    </script>
    <form id="op.logoutForm" method="post"
          action={{ do_logout }}>
        <input type="hidden" name="sjwt"
               value="{{ sjwt }}"/></form>
    <button onclick="logout()">Yes, sign me out</button>
    <button onclick="rpLogoutOnly()">No, stay signed in</button>
</div>
</body>
</html>