# Generated by Django 3.1.1 on 2021-05-16 16:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0007_auto_20210516_1230'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oidcsession',
            name='sid',
            field=models.CharField(
                blank=True, max_length=254, null=True, unique=True),
        ),
    ]
