# Generated by Django 3.1.1 on 2021-05-17 18:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0008_auto_20210516_1653'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oidcrpredirecturi',
            name='type',
            field=models.CharField(choices=[('redirect_uris', 'redirect_uris'), ('post_logout_redirect_uris', 'post_logout_redirect_uris'), (
                'frontchannel_logout_uri', 'frontchannel_logout_uri'), ('backchannel_logout_uri', 'backchannel_logout_uri')], max_length=33),
        ),
    ]
