# Generated by Django 3.1.1 on 2021-05-11 10:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0004_auto_20210511_0940'),
    ]

    operations = [
        migrations.AddField(
            model_name='oidcsession',
            name='sid_encrypted',
            field=models.CharField(blank=True, max_length=254, null=True),
        ),
    ]
