# Generated by Django 3.1.1 on 2021-05-11 09:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0003_auto_20210510_1353'),
    ]

    operations = [
        migrations.AddField(
            model_name='oidcsession',
            name='key',
            field=models.CharField(blank=True, max_length=254, null=True),
        ),
        migrations.AlterField(
            model_name='oidcsession',
            name='salt',
            field=models.CharField(blank=True, max_length=254, null=True),
        ),
        migrations.AlterField(
            model_name='oidcsession',
            name='sid',
            field=models.CharField(blank=True, max_length=254, null=True),
        ),
        migrations.AlterField(
            model_name='oidcsession',
            name='sub',
            field=models.CharField(blank=True, max_length=254, null=True),
        ),
    ]
